/*
 * Show the IP address of an interface.
 * usage: ./showip -i <interface> -t <seconds timeout>
 * e.g. ./showip -i eth0 -t 5
 *
 * based on code from:
 * -http://stackoverflow.com/questions/1570511/c-code-to-get-the-ip-address
 */

#define _GNU_SOURCE // for strnlen

#include <arpa/inet.h>
#include <sys/socket.h>
#include <netdb.h>
#include <ifaddrs.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "popup.h"

#define INTERFACE "eth0"
#define MAX_INTERFACE_LEN 4
#define MAX_IP_ADDR_STRING_LEN 15

/* 
 * get ip address of interface
 * iface: name of interface (eg. lo, eth0)
 * ip: string to be filled in with ip address
 * returns 0 on success
 */
static int get_ip(const char* iface, char *ip)
{
	struct ifaddrs *ifaddr, *ifa;
	int family, s;
	char host[NI_MAXHOST];

	if (getifaddrs(&ifaddr) == -1) {
		perror("getifaddrs");
		return -1;
	}

	for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
		family = ifa->ifa_addr->sa_family;

		if (family == AF_INET) {

			if (!strncmp(ifa->ifa_name, iface,
						strnlen(iface, MAX_INTERFACE_LEN))) {
				s = getnameinfo(ifa->ifa_addr, sizeof(struct sockaddr_in),
						host, NI_MAXHOST, NULL, 0, NI_NUMERICHOST);
				if (s != 0) {
					printf("getnameinfo() failed: %s\n", gai_strerror(s));
					return -1;
				}
				strncpy(ip, host, MAX_IP_ADDR_STRING_LEN);
				return 0;
			}
		}
	}

	fprintf(stderr, "interface %s not found.\n", iface);

	return -1;
}

void *print_message_function( void *ptr )
{
	char *message;
	message = (char *) ptr;
	printf("%s \n", message);
	return NULL;
}

int main(int argc, char *argv[])
{
	int c;
	char iface[MAX_INTERFACE_LEN+1];
	int timeout = 0;
	char my_ip[MAX_IP_ADDR_STRING_LEN+1];
	char text[MAX_INTERFACE_LEN + MAX_IP_ADDR_STRING_LEN + 10];

	opterr = 0;

	strncpy(iface, INTERFACE, MAX_INTERFACE_LEN);
	iface[MAX_INTERFACE_LEN] = '\0';

	while ((c = getopt(argc, argv, "i:t:")) != -1)
		switch (c) {
			case 'i':
				strncpy(iface, optarg, MAX_INTERFACE_LEN);
				break;
			case 't':
				timeout = atoi(optarg);
			default:
				break;
		}

	if ((timeout < 0) || (timeout > 60))
		timeout = 0;

	if (get_ip(iface, my_ip))
		return EXIT_FAILURE;

	my_ip[MAX_IP_ADDR_STRING_LEN] = '\0';
	sprintf(text, "%s: %s", iface, my_ip);
	printf("%s\n", text);

	if (display(text, timeout))
		return EXIT_FAILURE;

	return 0;
}
